@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
 
 #pragma once
 
@@ -41,10 +19,18 @@
 
 #include "qmljs/parser/qmljsmemorypool_p.h"
 
-#include <QtCore/qstring.h>
+#include <QtCore/qversionnumber.h>
+
+QT_BEGIN_NAMESPACE
+class QString;
+QT_END_NAMESPACE
 
 QT_QML_BEGIN_NAMESPACE
 
+namespace QmlJS {
+class Parser;
+}
+
 #define QMLJS_DECLARE_AST_NODE(name) \
   enum { K = Kind_##name };
 
@@ -201,6 +187,7 @@ public:
         Kind_SwitchStatement,
         Kind_TemplateLiteral,
         Kind_TaggedTemplate,
+        Kind_TypeExpression,
         Kind_ThisExpression,
         Kind_ThrowStatement,
         Kind_TildeExpression,
@@ -236,6 +223,7 @@ public:
         Kind_UiPragma,
         Kind_UiProgram,
         Kind_UiParameterList,
+        Kind_UiPropertyAttributes,
         Kind_UiPublicMember,
         Kind_UiQualifiedId,
         Kind_UiScriptBinding,
@@ -290,12 +278,6 @@ public:
             node->accept(visitor);
     }
 
-    // ### Remove when we can. This is part of the qmldevtools library, though.
-    inline static void acceptChild(Node *node, BaseVisitor *visitor)
-    {
-        return accept(node, visitor);
-    }
-
     virtual void accept0(BaseVisitor *visitor) = 0;
     virtual SourceLocation firstSourceLocation() const = 0;
     virtual SourceLocation lastSourceLocation() const = 0;
@@ -318,11 +300,11 @@ class QML_PARSER_EXPORT UiQualifiedId: public Node
 public:
     QMLJS_DECLARE_AST_NODE(UiQualifiedId)
 
-    UiQualifiedId(const QStringRef &name)
+    UiQualifiedId(QStringView name)
         : next(this), name(name)
     { kind = K; }
 
-    UiQualifiedId(UiQualifiedId *previous, const QStringRef &name)
+    UiQualifiedId(UiQualifiedId *previous, QStringView name)
         : name(name)
     {
         kind = K;
@@ -347,7 +329,7 @@ public:
 
 // attributes
     UiQualifiedId *next;
-    QStringRef name;
+    QStringView name;
     SourceLocation identifierToken;
 };
 
@@ -443,6 +425,7 @@ public:
     ExpressionNode() {}
 
     ExpressionNode *expressionCast() override;
+    bool containsOptionalChain() const;
 
     AST::FormalParameterList *reparseAsFormalParameterList(MemoryPool *pool);
 
@@ -488,6 +471,26 @@ public:
     SourceLocation rparenToken;
 };
 
+
+class QML_PARSER_EXPORT TypeExpression : public ExpressionNode
+{
+public:
+    QMLJS_DECLARE_AST_NODE(TypeExpression)
+    TypeExpression(Type *t) : m_type(t) { kind = K; }
+
+    void accept0(BaseVisitor *visitor) override;
+
+    SourceLocation firstSourceLocation() const override {
+        return m_type->firstSourceLocation();
+    }
+
+    SourceLocation lastSourceLocation() const override {
+        return m_type->lastSourceLocation();
+    }
+
+    Type *m_type;
+};
+
 class QML_PARSER_EXPORT ThisExpression: public LeftHandSideExpression
 {
 public:
@@ -512,7 +515,7 @@ class QML_PARSER_EXPORT IdentifierExpression: public LeftHandSideExpression
 public:
     QMLJS_DECLARE_AST_NODE(IdentifierExpression)
 
-    IdentifierExpression(const QStringRef &n):
+    IdentifierExpression(QStringView n):
         name (n) { kind = K; }
 
     void accept0(BaseVisitor *visitor) override;
@@ -524,7 +527,7 @@ public:
     { return identifierToken; }
 
 // attributes
-    QStringRef name;
+    QStringView name;
     SourceLocation identifierToken;
 };
 
@@ -631,7 +634,15 @@ class QML_PARSER_EXPORT UiVersionSpecifier : public Node
 public:
     QMLJS_DECLARE_AST_NODE(UiVersionSpecifier)
 
-    UiVersionSpecifier(int majorum, int minorum) : majorVersion(majorum), minorVersion(minorum) { kind = K; }
+    UiVersionSpecifier(int majorum) : majorVersion(majorum)
+    {
+        kind = K;
+    }
+
+    UiVersionSpecifier(int majorum, int minorum) : majorVersion(majorum), minorVersion(minorum)
+    {
+        kind = K;
+    }
 
     void accept0(BaseVisitor *visitor) override;
 
@@ -643,8 +654,8 @@ public:
     }
 
     // attributes:
-    int majorVersion;
-    int minorVersion;
+    int majorVersion = -1;
+    int minorVersion = -1;
     SourceLocation majorToken;
     SourceLocation minorToken;
 };
@@ -654,7 +665,7 @@ class QML_PARSER_EXPORT StringLiteral : public LeftHandSideExpression
 public:
     QMLJS_DECLARE_AST_NODE(StringLiteral)
 
-    StringLiteral(const QStringRef &v):
+    StringLiteral(QStringView v):
         value (v) { kind = K; }
 
     void accept0(BaseVisitor *visitor) override;
@@ -666,7 +677,7 @@ public:
     { return literalToken; }
 
 // attributes:
-    QStringRef value;
+    QStringView value;
     SourceLocation literalToken;
 };
 
@@ -675,7 +686,7 @@ class QML_PARSER_EXPORT TemplateLiteral : public LeftHandSideExpression
 public:
     QMLJS_DECLARE_AST_NODE(TemplateLiteral)
 
-    TemplateLiteral(const QStringRef &str, const QStringRef &raw, ExpressionNode *e)
+    TemplateLiteral(QStringView str, QStringView raw, ExpressionNode *e)
         : value(str), rawValue(raw), expression(e), next(nullptr)
     { kind = K; }
 
@@ -690,8 +701,9 @@ public:
 
     void accept0(BaseVisitor *visitor) override;
 
-    QStringRef value;
-    QStringRef rawValue;
+    bool hasNoSubstitution = false;
+    QStringView value;
+    QStringView rawValue;
     ExpressionNode *expression;
     TemplateLiteral *next;
     SourceLocation literalToken;
@@ -702,7 +714,7 @@ class QML_PARSER_EXPORT RegExpLiteral: public LeftHandSideExpression
 public:
     QMLJS_DECLARE_AST_NODE(RegExpLiteral)
 
-    RegExpLiteral(const QStringRef &p, int f):
+    RegExpLiteral(QStringView p, int f):
         pattern (p), flags (f) { kind = K; }
 
     void accept0(BaseVisitor *visitor) override;
@@ -714,7 +726,7 @@ public:
     { return literalToken; }
 
 // attributes:
-    QStringRef pattern;
+    QStringView pattern;
     int flags;
     SourceLocation literalToken;
 };
@@ -843,13 +855,20 @@ public:
 
 struct QML_PARSER_EXPORT BoundName
 {
+    enum Type {
+        Declared,
+        Injected,
+    };
+
     QString id;
-    TypeAnnotation *typeAnnotation = nullptr;
-    BoundName(const QString &id, TypeAnnotation *typeAnnotation)
-        : id(id), typeAnnotation(typeAnnotation)
+    TypeAnnotation *typeAnnotation;
+    Type typeAnnotationType;
+    BoundName(const QString &id, TypeAnnotation *typeAnnotation, Type type = Declared)
+    : id(id), typeAnnotation(typeAnnotation), typeAnnotationType(type)
     {}
     BoundName() = default;
     QString typeName() const { return typeAnnotation ? typeAnnotation->type->toString() : QString(); }
+    bool isInjected() const { return typeAnnotation && typeAnnotationType == Injected; }
 };
 
 struct BoundNames : public QVector<BoundName>
@@ -893,7 +912,7 @@ public:
         : initializer(i), type(t)
     { kind = K; }
 
-    PatternElement(const QStringRef &n, TypeAnnotation *typeAnnotation = nullptr, ExpressionNode *i = nullptr, Type t = Binding)
+    PatternElement(QStringView n, TypeAnnotation *typeAnnotation = nullptr, ExpressionNode *i = nullptr, Type t = Binding)
         : bindingIdentifier(n), initializer(i), type(t)
         , typeAnnotation(typeAnnotation)
     {
@@ -929,7 +948,7 @@ public:
 
 // attributes
     SourceLocation identifierToken;
-    QStringRef bindingIdentifier;
+    QStringView bindingIdentifier;
     ExpressionNode *bindingTarget = nullptr;
     ExpressionNode *initializer = nullptr;
     Type type = Literal;
@@ -937,6 +956,7 @@ public:
     // when used in a VariableDeclarationList
     VariableScope scope = VariableScope::NoScope;
     bool isForDeclaration = false;
+    bool isInjectedSignalParameter = false;
 };
 
 class QML_PARSER_EXPORT PatternElementList : public Node
@@ -988,7 +1008,7 @@ public:
         : PatternElement(i, t), name(name)
     { kind = K; }
 
-    PatternProperty(PropertyName *name, const QStringRef &n, ExpressionNode *i = nullptr)
+    PatternProperty(PropertyName *name, QStringView n, ExpressionNode *i = nullptr)
         : PatternElement(n, /*type annotation*/nullptr, i), name(name)
     { kind = K; }
 
@@ -1058,7 +1078,7 @@ class QML_PARSER_EXPORT IdentifierPropertyName: public PropertyName
 public:
     QMLJS_DECLARE_AST_NODE(IdentifierPropertyName)
 
-    IdentifierPropertyName(const QStringRef &n):
+    IdentifierPropertyName(QStringView n):
         id (n) { kind = K; }
 
     void accept0(BaseVisitor *visitor) override;
@@ -1066,7 +1086,7 @@ public:
     QString asString() const override { return id.toString(); }
 
 // attributes
-    QStringRef id;
+    QStringView id;
 };
 
 class QML_PARSER_EXPORT StringLiteralPropertyName: public PropertyName
@@ -1074,7 +1094,7 @@ class QML_PARSER_EXPORT StringLiteralPropertyName: public PropertyName
 public:
     QMLJS_DECLARE_AST_NODE(StringLiteralPropertyName)
 
-    StringLiteralPropertyName(const QStringRef &n):
+    StringLiteralPropertyName(QStringView n):
         id (n) { kind = K; }
 
     void accept0(BaseVisitor *visitor) override;
@@ -1082,7 +1102,7 @@ public:
     QString asString() const override { return id.toString(); }
 
 // attributes
-    QStringRef id;
+    QStringView id;
 };
 
 class QML_PARSER_EXPORT NumericLiteralPropertyName: public PropertyName
@@ -1147,6 +1167,7 @@ public:
     ExpressionNode *expression;
     SourceLocation lbracketToken;
     SourceLocation rbracketToken;
+    bool isOptional = false;
 };
 
 class QML_PARSER_EXPORT FieldMemberExpression: public LeftHandSideExpression
@@ -1154,7 +1175,7 @@ class QML_PARSER_EXPORT FieldMemberExpression: public LeftHandSideExpression
 public:
     QMLJS_DECLARE_AST_NODE(FieldMemberExpression)
 
-    FieldMemberExpression(ExpressionNode *b, const QStringRef &n):
+    FieldMemberExpression(ExpressionNode *b, QStringView n):
         base (b), name (n)
         { kind = K; }
 
@@ -1168,9 +1189,10 @@ public:
 
     // attributes
     ExpressionNode *base;
-    QStringRef name;
+    QStringView name;
     SourceLocation dotToken;
     SourceLocation identifierToken;
+    bool isOptional = false;
 };
 
 class QML_PARSER_EXPORT TaggedTemplate : public LeftHandSideExpression
@@ -1263,6 +1285,7 @@ public:
     ArgumentList *arguments;
     SourceLocation lparenToken;
     SourceLocation rparenToken;
+    bool isOptional = false;
 };
 
 class QML_PARSER_EXPORT ArgumentList: public Node
@@ -1944,7 +1967,7 @@ class QML_PARSER_EXPORT ContinueStatement: public Statement
 public:
     QMLJS_DECLARE_AST_NODE(ContinueStatement)
 
-    ContinueStatement(const QStringRef &l = QStringRef()):
+    ContinueStatement(QStringView l = QStringView()):
         label (l) { kind = K; }
 
     void accept0(BaseVisitor *visitor) override;
@@ -1956,7 +1979,7 @@ public:
     { return semicolonToken; }
 
 // attributes
-    QStringRef label;
+    QStringView label;
     SourceLocation continueToken;
     SourceLocation identifierToken;
     SourceLocation semicolonToken;
@@ -1967,7 +1990,7 @@ class QML_PARSER_EXPORT BreakStatement: public Statement
 public:
     QMLJS_DECLARE_AST_NODE(BreakStatement)
 
-    BreakStatement(const QStringRef &l):
+    BreakStatement(QStringView l):
         label (l) { kind = K; }
 
     void accept0(BaseVisitor *visitor) override;
@@ -1979,7 +2002,7 @@ public:
     { return semicolonToken; }
 
     // attributes
-    QStringRef label;
+    QStringView label;
     SourceLocation breakToken;
     SourceLocation identifierToken;
     SourceLocation semicolonToken;
@@ -2195,7 +2218,7 @@ class QML_PARSER_EXPORT LabelledStatement: public Statement
 public:
     QMLJS_DECLARE_AST_NODE(LabelledStatement)
 
-    LabelledStatement(const QStringRef &l, Statement *stmt):
+    LabelledStatement(QStringView l, Statement *stmt):
         label (l), statement (stmt)
         { kind = K; }
 
@@ -2208,7 +2231,7 @@ public:
     { return statement->lastSourceLocation(); }
 
 // attributes
-    QStringRef label;
+    QStringView label;
     Statement *statement;
     SourceLocation identifierToken;
     SourceLocation colonToken;
@@ -2328,7 +2351,7 @@ class QML_PARSER_EXPORT FunctionExpression: public ExpressionNode
 public:
     QMLJS_DECLARE_AST_NODE(FunctionExpression)
 
-    FunctionExpression(const QStringRef &n, FormalParameterList *f, StatementList *b, TypeAnnotation *typeAnnotation = nullptr):
+    FunctionExpression(QStringView n, FormalParameterList *f, StatementList *b, TypeAnnotation *typeAnnotation = nullptr):
         name (n), formals (f), body (b),
         typeAnnotation(typeAnnotation)
         { kind = K; }
@@ -2344,7 +2367,7 @@ public:
     FunctionExpression *asFunctionDefinition() override;
 
 // attributes
-    QStringRef name;
+    QStringView name;
     bool isArrowFunction = false;
     bool isGenerator = false;
     FormalParameterList *formals;
@@ -2363,7 +2386,7 @@ class QML_PARSER_EXPORT FunctionDeclaration: public FunctionExpression
 public:
     QMLJS_DECLARE_AST_NODE(FunctionDeclaration)
 
-    FunctionDeclaration(const QStringRef &n, FormalParameterList *f, StatementList *b, TypeAnnotation *typeAnnotation = nullptr):
+    FunctionDeclaration(QStringView n, FormalParameterList *f, StatementList *b, TypeAnnotation *typeAnnotation = nullptr):
         FunctionExpression(n, f, b, typeAnnotation)
         { kind = K; }
 
@@ -2460,7 +2483,7 @@ class QML_PARSER_EXPORT ClassExpression : public ExpressionNode
 public:
     QMLJS_DECLARE_AST_NODE(ClassExpression)
 
-    ClassExpression(const QStringRef &n, ExpressionNode *heritage, ClassElementList *elements)
+    ClassExpression(QStringView n, ExpressionNode *heritage, ClassElementList *elements)
         : name(n), heritage(heritage), elements(elements)
         { kind = K; }
 
@@ -2475,7 +2498,7 @@ public:
     ClassExpression *asClassDefinition() override;
 
 // attributes
-    QStringRef name;
+    QStringView name;
     ExpressionNode *heritage;
     ClassElementList *elements;
     SourceLocation classToken;
@@ -2489,7 +2512,7 @@ class QML_PARSER_EXPORT ClassDeclaration: public ClassExpression
 public:
     QMLJS_DECLARE_AST_NODE(ClassDeclaration)
 
-    ClassDeclaration(const QStringRef &n, ExpressionNode *heritage, ClassElementList *elements)
+    ClassDeclaration(QStringView n, ExpressionNode *heritage, ClassElementList *elements)
         : ClassExpression(n, heritage, elements)
         { kind = K; }
 
@@ -2560,13 +2583,13 @@ class QML_PARSER_EXPORT ImportSpecifier: public Node
 public:
     QMLJS_DECLARE_AST_NODE(ImportSpecifier)
 
-    ImportSpecifier(const QStringRef &importedBinding)
+    ImportSpecifier(QStringView importedBinding)
         : importedBinding(importedBinding)
     {
         kind = K;
     }
 
-    ImportSpecifier(const QStringRef &identifier, const QStringRef &importedBinding)
+    ImportSpecifier(QStringView identifier, QStringView importedBinding)
         : identifier(identifier), importedBinding(importedBinding)
     {
         kind = K;
@@ -2582,8 +2605,8 @@ public:
 // attributes
     SourceLocation identifierToken;
     SourceLocation importedBindingToken;
-    QStringRef identifier;
-    QStringRef importedBinding;
+    QStringView identifier;
+    QStringView importedBinding;
 };
 
 class QML_PARSER_EXPORT ImportsList: public Node
@@ -2667,7 +2690,7 @@ class QML_PARSER_EXPORT NameSpaceImport: public Node
 public:
     QMLJS_DECLARE_AST_NODE(NameSpaceImport)
 
-    NameSpaceImport(const QStringRef &importedBinding)
+    NameSpaceImport(QStringView importedBinding)
         : importedBinding(importedBinding)
     {
         kind = K;
@@ -2683,7 +2706,7 @@ public:
 // attributes
     SourceLocation starToken;
     SourceLocation importedBindingToken;
-    QStringRef importedBinding;
+    QStringView importedBinding;
 };
 
 class QML_PARSER_EXPORT ImportClause: public Node
@@ -2691,7 +2714,7 @@ class QML_PARSER_EXPORT ImportClause: public Node
 public:
     QMLJS_DECLARE_AST_NODE(ImportClause)
 
-    ImportClause(const QStringRef &importedDefaultBinding)
+    ImportClause(QStringView importedDefaultBinding)
         : importedDefaultBinding(importedDefaultBinding)
     {
         kind = K;
@@ -2709,14 +2732,14 @@ public:
         kind = K;
     }
 
-    ImportClause(const QStringRef &importedDefaultBinding, NameSpaceImport *nameSpaceImport)
+    ImportClause(QStringView importedDefaultBinding, NameSpaceImport *nameSpaceImport)
         : importedDefaultBinding(importedDefaultBinding)
         , nameSpaceImport(nameSpaceImport)
     {
         kind = K;
     }
 
-    ImportClause(const QStringRef &importedDefaultBinding, NamedImports *namedImports)
+    ImportClause(QStringView importedDefaultBinding, NamedImports *namedImports)
         : importedDefaultBinding(importedDefaultBinding)
         , namedImports(namedImports)
     {
@@ -2732,7 +2755,7 @@ public:
 
 // attributes
     SourceLocation importedDefaultBindingToken;
-    QStringRef importedDefaultBinding;
+    QStringView importedDefaultBinding;
     NameSpaceImport *nameSpaceImport = nullptr;
     NamedImports *namedImports = nullptr;
 };
@@ -2742,7 +2765,7 @@ class QML_PARSER_EXPORT FromClause: public Node
 public:
     QMLJS_DECLARE_AST_NODE(FromClause)
 
-    FromClause(const QStringRef &moduleSpecifier)
+    FromClause(QStringView moduleSpecifier)
         : moduleSpecifier(moduleSpecifier)
     {
         kind = K;
@@ -2759,7 +2782,7 @@ public:
 // attributes
     SourceLocation fromToken;
     SourceLocation moduleSpecifierToken;
-    QStringRef moduleSpecifier;
+    QStringView moduleSpecifier;
 };
 
 class QML_PARSER_EXPORT ImportDeclaration: public Statement
@@ -2773,7 +2796,7 @@ public:
         kind = K;
     }
 
-    ImportDeclaration(const QStringRef &moduleSpecifier)
+    ImportDeclaration(QStringView moduleSpecifier)
         : moduleSpecifier(moduleSpecifier)
     {
         kind = K;
@@ -2790,7 +2813,7 @@ public:
 // attributes
     SourceLocation importToken;
     SourceLocation moduleSpecifierToken;
-    QStringRef moduleSpecifier;
+    QStringView moduleSpecifier;
     ImportClause *importClause = nullptr;
     FromClause *fromClause = nullptr;
 };
@@ -2800,13 +2823,13 @@ class QML_PARSER_EXPORT ExportSpecifier: public Node
 public:
     QMLJS_DECLARE_AST_NODE(ExportSpecifier)
 
-    ExportSpecifier(const QStringRef &identifier)
+    ExportSpecifier(QStringView identifier)
         : identifier(identifier), exportedIdentifier(identifier)
     {
         kind = K;
     }
 
-    ExportSpecifier(const QStringRef &identifier, const QStringRef &exportedIdentifier)
+    ExportSpecifier(QStringView identifier, QStringView exportedIdentifier)
         : identifier(identifier), exportedIdentifier(exportedIdentifier)
     {
         kind = K;
@@ -2822,8 +2845,8 @@ public:
 // attributes
     SourceLocation identifierToken;
     SourceLocation exportedIdentifierToken;
-    QStringRef identifier;
-    QStringRef exportedIdentifier;
+    QStringView identifier;
+    QStringView exportedIdentifier;
 };
 
 class QML_PARSER_EXPORT ExportsList: public Node
@@ -2906,7 +2929,6 @@ public:
     ExportDeclaration(FromClause *fromClause)
         : fromClause(fromClause)
     {
-        exportAll = true;
         kind = K;
     }
 
@@ -2929,6 +2951,11 @@ public:
         kind = K;
     }
 
+    bool exportsAll() const
+    {
+        return fromClause && !exportClause;
+    }
+
     void accept0(BaseVisitor *visitor) override;
 
     SourceLocation firstSourceLocation() const override
@@ -2938,7 +2965,6 @@ public:
 
 // attributes
     SourceLocation exportToken;
-    bool exportAll = false;
     ExportClause *exportClause = nullptr;
     FromClause *fromClause = nullptr;
     Node *variableStatementOrDeclaration = nullptr;
@@ -2994,7 +3020,7 @@ class QML_PARSER_EXPORT UiImport: public Node
 public:
     QMLJS_DECLARE_AST_NODE(UiImport)
 
-    UiImport(const QStringRef &fileName)
+    UiImport(QStringView fileName)
         : fileName(fileName), importUri(nullptr)
     { kind = K; }
 
@@ -3011,9 +3037,9 @@ public:
     { return semicolonToken; }
 
 // attributes
-    QStringRef fileName;
+    QStringView fileName;
     UiQualifiedId *importUri;
-    QStringRef importId;
+    QStringView importId;
     SourceLocation importToken;
     SourceLocation fileNameToken;
     SourceLocation asToken;
@@ -3076,8 +3102,9 @@ class QML_PARSER_EXPORT UiPragma: public Node
 public:
     QMLJS_DECLARE_AST_NODE(UiPragma)
 
-    UiPragma(QStringRef name)
+    UiPragma(QStringView name, QStringView value = {})
         : name(name)
+        , value(value)
     { kind = K; }
 
     void accept0(BaseVisitor *visitor) override;
@@ -3089,7 +3116,8 @@ public:
     { return semicolonToken; }
 
 // attributes
-    QStringRef name;
+    QStringView name;
+    QStringView value;
     SourceLocation pragmaToken;
     SourceLocation semicolonToken;
 };
@@ -3099,7 +3127,7 @@ class QML_PARSER_EXPORT UiRequired: public Node
 public:
     QMLJS_DECLARE_AST_NODE(UiRequired)
 
-    UiRequired(QStringRef name)
+    UiRequired(QStringView name)
         :name(name)
     { kind = K; }
 
@@ -3111,7 +3139,7 @@ public:
     SourceLocation lastSourceLocation() const override
     { return semicolonToken; }
 
-    QStringRef name;
+    QStringView name;
     SourceLocation requiredToken;
     SourceLocation semicolonToken;
 };
@@ -3265,11 +3293,11 @@ class QML_PARSER_EXPORT UiParameterList: public Node
 public:
     QMLJS_DECLARE_AST_NODE(UiParameterList)
 
-    UiParameterList(UiQualifiedId *t, const QStringRef &n):
+    UiParameterList(UiQualifiedId *t, QStringView n):
         type (t), name (n), next (this)
         { kind = K; }
 
-    UiParameterList(UiParameterList *previous, UiQualifiedId *t, const QStringRef &n):
+    UiParameterList(UiParameterList *previous, UiQualifiedId *t, QStringView n):
         type (t), name (n)
     {
         kind = K;
@@ -3297,7 +3325,7 @@ public:
 
 // attributes
     UiQualifiedId *type;
-    QStringRef name;
+    QStringView name;
     UiParameterList *next;
     SourceLocation commaToken;
     SourceLocation propertyTypeToken;
@@ -3305,37 +3333,77 @@ public:
     SourceLocation colonToken;
 };
 
+class QML_PARSER_EXPORT UiPropertyAttributes : public Node
+{
+    QMLJS_DECLARE_AST_NODE(UiPropertyAttributes)
+public:
+    UiPropertyAttributes() { kind = K; }
+
+    SourceLocation defaultToken() const { return m_defaultToken; }
+    bool isDefaultMember() const { return defaultToken().isValid(); }
+    SourceLocation requiredToken() const { return m_requiredToken; }
+    bool isRequired() const { return requiredToken().isValid(); }
+    SourceLocation readonlyToken() const { return m_readonlyToken; }
+    bool isReadonly() const { return readonlyToken().isValid(); }
+
+    SourceLocation propertyToken() const { return m_propertyToken; }
+
+    template<bool InvalidIsLargest = true>
+    static bool compareLocationsByBegin(const SourceLocation *&lhs, const SourceLocation *&rhs)
+    {
+        if (lhs->isValid() && rhs->isValid())
+            return lhs->begin() < rhs->begin();
+        else if (lhs->isValid())
+            return InvalidIsLargest;
+        else
+            return !InvalidIsLargest;
+    }
+
+    void accept0(BaseVisitor *) override {} // intentionally do nothing
+
+    SourceLocation firstSourceLocation() const override;
+
+    SourceLocation lastSourceLocation() const override;
+
+private:
+    friend class QmlJS::Parser;
+    SourceLocation m_defaultToken;
+    SourceLocation m_readonlyToken;
+    SourceLocation m_requiredToken;
+    SourceLocation m_propertyToken;
+};
+
 class QML_PARSER_EXPORT UiPublicMember: public UiObjectMember
 {
 public:
     QMLJS_DECLARE_AST_NODE(UiPublicMember)
 
-    UiPublicMember(UiQualifiedId *memberType,
-                   const QStringRef &name)
-        : type(Property), memberType(memberType), name(name), statement(nullptr), binding(nullptr), isDefaultMember(false), isReadonlyMember(false), parameters(nullptr)
+    UiPublicMember(UiQualifiedId *memberType, QStringView name)
+        : type(Property)
+        , memberType(memberType)
+        , name(name)
+        , statement(nullptr)
+        , binding(nullptr)
+        , parameters(nullptr)
     { kind = K; }
 
-    UiPublicMember(UiQualifiedId *memberType,
-                   const QStringRef &name,
-                   Statement *statement)
-        : type(Property), memberType(memberType), name(name), statement(statement), binding(nullptr), isDefaultMember(false), isReadonlyMember(false), parameters(nullptr)
+    UiPublicMember(UiQualifiedId *memberType, QStringView name, Statement *statement)
+        : type(Property)
+        , memberType(memberType)
+        , name(name)
+        , statement(statement)
+        , binding(nullptr)
+        , parameters(nullptr)
     { kind = K; }
 
     void accept0(BaseVisitor *visitor) override;
 
     SourceLocation firstSourceLocation() const override
     {
-        if (requiredToken.isValid()) {
-            if (defaultToken.isValid() && defaultToken.offset < requiredToken.offset)
-                return defaultToken;
-            return requiredToken;
-        }
-        if (defaultToken.isValid())
-            return defaultToken;
-        if (readonlyToken.isValid())
-            return readonlyToken;
-
-        return propertyToken;
+        if (hasAttributes)
+            return m_attributes->firstSourceLocation();
+        else
+            return m_propertyToken;
     }
 
     SourceLocation lastSourceLocation() const override
@@ -3348,27 +3416,62 @@ public:
       return semicolonToken;
     }
 
+    SourceLocation defaultToken() const
+    {
+        return hasAttributes ? m_attributes->defaultToken() : SourceLocation{};
+    }
+    bool isDefaultMember() const { return defaultToken().isValid(); }
+
+    SourceLocation requiredToken() const
+    {
+        return hasAttributes ? m_attributes->requiredToken() : SourceLocation{};
+    }
+    bool isRequired() const { return requiredToken().isValid(); }
+
+    SourceLocation readonlyToken() const
+    {
+        return hasAttributes ? m_attributes->readonlyToken() : SourceLocation{};
+    }
+    bool isReadonly() const { return readonlyToken().isValid(); }
+
+    void setAttributes(UiPropertyAttributes *attributes)
+    {
+        m_attributes = attributes;
+        hasAttributes = true;
+    }
+
+    SourceLocation propertyToken() const
+    {
+        return hasAttributes ? m_attributes->propertyToken() : m_propertyToken;
+    }
+
+    void setPropertyToken(SourceLocation token)
+    {
+        m_propertyToken = token;
+        hasAttributes = false;
+    }
+
 // attributes
-    enum { Signal, Property } type;
-    QStringRef typeModifier;
+    enum : bool { Signal, Property } type;
+    bool hasAttributes = false;
+    QStringView typeModifier;
     UiQualifiedId *memberType;
-    QStringRef name;
+    QStringView name;
     Statement *statement; // initialized with a JS expression
     UiObjectMember *binding; // initialized with a QML object or array.
-    bool isDefaultMember;
-    bool isReadonlyMember;
-    bool isRequired = false;
     UiParameterList *parameters;
     // TODO: merge source locations
-    SourceLocation defaultToken;
-    SourceLocation readonlyToken;
-    SourceLocation propertyToken;
-    SourceLocation requiredToken;
     SourceLocation typeModifierToken;
     SourceLocation typeToken;
     SourceLocation identifierToken;
     SourceLocation colonToken;
     SourceLocation semicolonToken;
+
+private:
+    union {
+        SourceLocation m_propertyToken = SourceLocation{};
+        UiPropertyAttributes *m_attributes;
+    };
 };
 
 class QML_PARSER_EXPORT UiObjectDefinition: public UiObjectMember
@@ -3399,7 +3502,7 @@ class QML_PARSER_EXPORT UiInlineComponent: public UiObjectMember
 public:
     QMLJS_DECLARE_AST_NODE(UiInlineComponent)
 
-    UiInlineComponent(const QStringRef& inlineComponentName, UiObjectDefinition* inlineComponent)
+    UiInlineComponent(QStringView inlineComponentName, UiObjectDefinition* inlineComponent)
         : name(inlineComponentName), component(inlineComponent)
     { kind = K; }
 
@@ -3412,7 +3515,7 @@ public:
     void accept0(BaseVisitor *visitor) override;
 
     // attributes
-    QStringRef name;
+    QStringView name;
     UiObjectDefinition* component;
     SourceLocation componentToken;
 };
@@ -3526,7 +3629,7 @@ public:
     { kind = K; }
 
     SourceLocation firstSourceLocation() const override
-    { return qualifiedId->identifierToken; }
+    { Q_ASSERT(qualifiedId); return qualifiedId->identifierToken; }
 
     SourceLocation lastSourceLocation() const override
     { return rbracketToken; }
@@ -3545,11 +3648,11 @@ class QML_PARSER_EXPORT UiEnumMemberList: public Node
 {
     QMLJS_DECLARE_AST_NODE(UiEnumMemberList)
 public:
-    UiEnumMemberList(const QStringRef &member, double v = 0.0)
+    UiEnumMemberList(QStringView member, double v = 0.0)
         : next(this), member(member), value(v)
     { kind = K; }
 
-    UiEnumMemberList(UiEnumMemberList *previous, const QStringRef &member)
+    UiEnumMemberList(UiEnumMemberList *previous, QStringView member)
         : member(member)
     {
         kind = K;
@@ -3558,7 +3661,7 @@ public:
         value = previous->value + 1;
     }
 
-    UiEnumMemberList(UiEnumMemberList *previous, const QStringRef &member, double v)
+    UiEnumMemberList(UiEnumMemberList *previous, QStringView member, double v)
         : member(member), value(v)
     {
         kind = K;
@@ -3586,7 +3689,7 @@ public:
 
 // attributes
     UiEnumMemberList *next;
-    QStringRef member;
+    QStringView member;
     double value;
     SourceLocation memberToken;
     SourceLocation valueToken;
@@ -3597,7 +3700,7 @@ class QML_PARSER_EXPORT UiEnumDeclaration: public UiObjectMember
 public:
     QMLJS_DECLARE_AST_NODE(UiEnumDeclaration)
 
-    UiEnumDeclaration(const QStringRef &name,
+    UiEnumDeclaration(QStringView name,
                       UiEnumMemberList *members)
         : name(name)
         , members(members)
@@ -3614,7 +3717,7 @@ public:
 // attributes
     SourceLocation enumToken;
     SourceLocation rbraceToken;
-    QStringRef name;
+    QStringView name;
     UiEnumMemberList *members;
 };
 
@@ -3682,4 +3785,3 @@ public:
 
 
 QT_QML_END_NAMESPACE
-
