@@ -1,36 +1,9 @@
-/****************************************************************************
-**
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
 
 #include "glslcompletionassist.h"
 #include "glsleditorconstants.h"
 #include "glsleditorplugin.h"
-#include "reuse.h"
 
 #include <glsl/glslengine.h>
 #include <glsl/glsllexer.h>
@@ -41,14 +14,16 @@
 
 #include <coreplugin/idocument.h>
 #include <texteditor/completionsettings.h>
-#include <texteditor/codeassist/basicproposalitem.h>
-#include <texteditor/codeassist/basicproposalitemlistmodel.h>
+#include <texteditor/codeassist/assistproposalitem.h>
+#include <texteditor/codeassist/genericproposalmodel.h>
 #include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/functionhintproposal.h>
+#include <texteditor/texteditorsettings.h>
 #include <cplusplus/ExpressionUnderCursor.h>
+#include <cplusplus/Icons.h>
 
+#include <utils/icon.h>
 #include <utils/faketooltip.h>
-#include <utils/qtcoverride.h>
 
 #include <QIcon>
 #include <QPainter>
@@ -56,31 +31,52 @@
 #include <QToolButton>
 #include <QHBoxLayout>
 #include <QApplication>
-#include <QDesktopWidget>
 #include <QDebug>
 
-using namespace GLSLEditor;
-using namespace Internal;
 using namespace TextEditor;
 
-namespace {
+namespace GlslEditor {
+namespace Internal {
+
+Document::~Document()
+{
+    delete _globalScope;
+    delete _engine;
+}
+
+GLSL::Scope *Document::scopeAt(int position) const
+{
+    foreach (const Range &c, _cursors) {
+        if (position >= c.cursor.selectionStart() && position <= c.cursor.selectionEnd())
+            return c.scope;
+    }
+    return _globalScope;
+}
+
+void Document::addRange(const QTextCursor &cursor, GLSL::Scope *scope)
+{
+    Range c;
+    c.cursor = cursor;
+    c.scope = scope;
+    _cursors.append(c);
+}
+
 
 enum CompletionOrder {
     SpecialMemberOrder = -5
 };
 
-
-bool isActivationChar(const QChar &ch)
+static bool isActivationChar(const QChar &ch)
 {
     return ch == QLatin1Char('(') || ch == QLatin1Char('.') || ch == QLatin1Char(',');
 }
 
-bool isIdentifierChar(QChar ch)
+static bool isIdentifierChar(QChar ch)
 {
     return ch.isLetterOrNumber() || ch == QLatin1Char('_');
 }
 
-bool isDelimiter(QChar ch)
+static bool isDelimiter(QChar ch)
 {
     switch (ch.unicode()) {
     case '{':
@@ -104,7 +100,7 @@ bool isDelimiter(QChar ch)
     }
 }
 
-bool checkStartOfIdentifier(const QString &word)
+static bool checkStartOfIdentifier(const QString &word)
 {
     if (! word.isEmpty()) {
         const QChar ch = word.at(0);
@@ -115,63 +111,146 @@ bool checkStartOfIdentifier(const QString &word)
     return false;
 }
 
-} // Anonymous
+enum IconTypes {
+    IconTypeAttribute,
+    IconTypeUniform,
+    IconTypeKeyword,
+    IconTypeVarying,
+    IconTypeConst,
+    IconTypeVariable,
+    IconTypeType,
+    IconTypeFunction,
+    IconTypeOther
+};
 
-// ----------------------------
-// GLSLCompletionAssistProvider
-// ----------------------------
-bool GLSLCompletionAssistProvider::supportsEditor(Core::Id editorId) const
+static QIcon glslIcon(IconTypes iconType)
 {
-    return editorId == Constants::C_GLSLEDITOR_ID;
+    using namespace CPlusPlus;
+    using namespace Utils;
+
+    const char member[] = ":/codemodel/images/member.png";
+
+    switch (iconType) {
+    case IconTypeType:
+        return Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::Class);
+    case IconTypeConst:
+        return Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::Enumerator);
+    case IconTypeKeyword:
+        return Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::Keyword);
+    case IconTypeFunction:
+        return Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::FuncPublic);
+    case IconTypeVariable:
+        return Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::VarPublic);
+    case IconTypeAttribute: {
+        static const QIcon icon =
+                Icon({{member, Theme::IconsCodeModelAttributeColor}}, Icon::Tint).icon();
+        return icon;
+    }
+    case IconTypeUniform: {
+        static const QIcon icon =
+                Icon({{member, Theme::IconsCodeModelUniformColor}}, Icon::Tint).icon();
+        return icon;
+    }
+    case IconTypeVarying: {
+        static const QIcon icon =
+                Icon({{member, Theme::IconsCodeModelVaryingColor}}, Icon::Tint).icon();
+        return icon;
+    }
+    case IconTypeOther:
+    default:
+        return Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::Namespace);
+    }
 }
 
-IAssistProcessor *GLSLCompletionAssistProvider::createProcessor() const
+// ----------------------------
+// GlslCompletionAssistProvider
+// ----------------------------
+IAssistProcessor *GlslCompletionAssistProvider::createProcessor(const AssistInterface *) const
 {
-    return new GLSLCompletionAssistProcessor;
+    return new GlslCompletionAssistProcessor;
 }
 
-int GLSLCompletionAssistProvider::activationCharSequenceLength() const
+int GlslCompletionAssistProvider::activationCharSequenceLength() const
 {
     return 1;
 }
 
-bool GLSLCompletionAssistProvider::isActivationCharSequence(const QString &sequence) const
+bool GlslCompletionAssistProvider::isActivationCharSequence(const QString &sequence) const
 {
     return isActivationChar(sequence.at(0));
 }
 
+struct FunctionItem
+{
+    FunctionItem() = default;
+    explicit FunctionItem(const GLSL::Function *function);
+    QString prettyPrint(int currentArgument) const;
+    QString returnValue;
+    QString name;
+    QStringList argsWithType;
+};
+
+FunctionItem::FunctionItem(const GLSL::Function *function)
+{
+    Q_ASSERT(function);
+    returnValue = function->returnType()->toString();
+    name = function->name();
+    for (auto arg : function->arguments())
+        argsWithType.append(arg->type()->toString() + QLatin1Char(' ') + arg->name());
+}
+
+QString FunctionItem::prettyPrint(int currentArgument) const
+{
+    QString result = returnValue + QLatin1Char(' ') + name + QLatin1Char('(');
+    for (int i = 0; i < argsWithType.size(); ++i) {
+        if (i != 0)
+            result += QLatin1String(", ");
+        if (currentArgument == i)
+            result += QLatin1String("<b>");
+        result += argsWithType.at(i);
+        if (currentArgument == i)
+            result += QLatin1String("</b>");
+    }
+    result += QLatin1Char(')');
+    return result;
+}
+
 // -----------------------------
-// GLSLFunctionHintProposalModel
+// GlslFunctionHintProposalModel
 // -----------------------------
-class GLSLFunctionHintProposalModel : public TextEditor::IFunctionHintProposalModel
+class GlslFunctionHintProposalModel : public IFunctionHintProposalModel
 {
 public:
-    GLSLFunctionHintProposalModel(QVector<GLSL::Function *> functionSymbols)
-        : m_items(functionSymbols)
-        , m_currentArg(-1)
-    {}
+    GlslFunctionHintProposalModel(QVector<GLSL::Function *> functionSymbols);
 
-    void reset() QTC_OVERRIDE {}
-    int size() const QTC_OVERRIDE { return m_items.size(); }
-    QString text(int index) const QTC_OVERRIDE;
-    int activeArgument(const QString &prefix) const QTC_OVERRIDE;
+    void reset() override {}
+    int size() const override { return m_items.size(); }
+    QString text(int index) const override;
+    int activeArgument(const QString &prefix) const override;
 
 private:
-    QVector<GLSL::Function *> m_items;
+    QVector<FunctionItem> m_items;
     mutable int m_currentArg;
 };
 
-QString GLSLFunctionHintProposalModel::text(int index) const
+GlslFunctionHintProposalModel::GlslFunctionHintProposalModel(QVector<GLSL::Function *> symbols)
+    : m_currentArg(-1)
 {
-    return m_items.at(index)->prettyPrint(m_currentArg);
+    for (const GLSL::Function *symbol : symbols)
+        m_items.append(FunctionItem(symbol));
 }
 
-int GLSLFunctionHintProposalModel::activeArgument(const QString &prefix) const
+QString GlslFunctionHintProposalModel::text(int index) const
+{
+    return m_items.at(index).prettyPrint(m_currentArg);
+}
+
+int GlslFunctionHintProposalModel::activeArgument(const QString &prefix) const
 {
     const QByteArray &str = prefix.toLatin1();
     int argnr = 0;
     int parcount = 0;
-    GLSL::Lexer lexer(0, str.constData(), str.length());
+    GLSL::Lexer lexer(nullptr, str.constData(), str.length());
     GLSL::Token tk;
     QList<GLSL::Token> tokens;
     do {
@@ -200,39 +279,36 @@ int GLSLFunctionHintProposalModel::activeArgument(const QString &prefix) const
 // -----------------------------
 // GLSLCompletionAssistProcessor
 // -----------------------------
-GLSLCompletionAssistProcessor::GLSLCompletionAssistProcessor()
-    : m_startPosition(0)
-    , m_keywordIcon(QLatin1String(":/glsleditor/images/keyword.png"))
-    , m_varIcon(QLatin1String(":/glsleditor/images/var.png"))
-    , m_functionIcon(QLatin1String(":/glsleditor/images/func.png"))
-    , m_typeIcon(QLatin1String(":/glsleditor/images/type.png"))
-    , m_constIcon(QLatin1String(":/glsleditor/images/const.png"))
-    , m_attributeIcon(QLatin1String(":/glsleditor/images/attribute.png"))
-    , m_uniformIcon(QLatin1String(":/glsleditor/images/uniform.png"))
-    , m_varyingIcon(QLatin1String(":/glsleditor/images/varying.png"))
-    , m_otherIcon(QLatin1String(":/glsleditor/images/other.png"))
-{}
+GlslCompletionAssistProcessor::~GlslCompletionAssistProcessor() = default;
 
-GLSLCompletionAssistProcessor::~GLSLCompletionAssistProcessor()
-{}
-
-IAssistProposal *GLSLCompletionAssistProcessor::perform(const IAssistInterface *interface)
+static AssistProposalItem *createCompletionItem(const QString &text, const QIcon &icon, int order = 0)
 {
-    m_interface.reset(static_cast<const GLSLCompletionAssistInterface *>(interface));
+    auto item = new AssistProposalItem;
+    item->setText(text);
+    item->setIcon(icon);
+    item->setOrder(order);
+    return item;
+}
+
+IAssistProposal *GlslCompletionAssistProcessor::perform(const AssistInterface *interface)
+{
+    m_interface.reset(static_cast<const GlslCompletionAssistInterface *>(interface));
 
     if (interface->reason() == IdleEditor && !acceptsIdleEditor())
-        return 0;
+        return nullptr;
 
     int pos = m_interface->position() - 1;
     QChar ch = m_interface->characterAt(pos);
     while (ch.isLetterOrNumber() || ch == QLatin1Char('_'))
         ch = m_interface->characterAt(--pos);
 
-    CPlusPlus::ExpressionUnderCursor expressionUnderCursor;
+    CPlusPlus::ExpressionUnderCursor expressionUnderCursor(
+                CPlusPlus::LanguageFeatures::defaultFeatures());
     //GLSLTextEditorWidget *edit = qobject_cast<GLSLTextEditorWidget *>(editor->widget());
 
     QList<GLSL::Symbol *> members;
     QStringList specialMembers;
+    QList<AssistProposalItemInterface *> m_completions;
 
     bool functionCall = (ch == QLatin1Char('(') && pos == m_interface->position() - 1);
 
@@ -241,7 +317,7 @@ IAssistProposal *GLSLCompletionAssistProcessor::perform(const IAssistInterface *
         tc.setPosition(pos);
         const int start = expressionUnderCursor.startOfFunctionCall(tc);
         if (start == -1)
-            return 0;
+            return nullptr;
 
         if (m_interface->characterAt(start) == QLatin1Char('(')) {
             pos = start;
@@ -338,7 +414,7 @@ IAssistProposal *GLSLCompletionAssistProcessor::perform(const IAssistInterface *
                     "qt_MultiTexCoord0",
                     "qt_MultiTexCoord1",
                     "qt_MultiTexCoord2",
-                    0
+                    nullptr
                 };
                 static const char * const uniformNames[] = {
                     "qt_ModelViewProjectionMatrix",
@@ -350,12 +426,12 @@ IAssistProposal *GLSLCompletionAssistProcessor::perform(const IAssistInterface *
                     "qt_Texture2",
                     "qt_Color",
                     "qt_Opacity",
-                    0
+                    nullptr
                 };
                 for (int index = 0; attributeNames[index]; ++index)
-                    addCompletion(QString::fromLatin1(attributeNames[index]), m_attributeIcon);
+                    m_completions << createCompletionItem(QString::fromLatin1(attributeNames[index]), glslIcon(IconTypeAttribute));
                 for (int index = 0; uniformNames[index]; ++index)
-                    addCompletion(QString::fromLatin1(uniformNames[index]), m_uniformIcon);
+                    m_completions << createCompletionItem(QString::fromLatin1(uniformNames[index]), glslIcon(IconTypeUniform));
             }
         }
 
@@ -363,7 +439,7 @@ IAssistProposal *GLSLCompletionAssistProcessor::perform(const IAssistInterface *
             QStringList keywords = GLSL::Lexer::keywords(languageVariant(m_interface->mimeType()));
 //            m_keywordCompletions.clear();
             for (int index = 0; index < keywords.size(); ++index)
-                addCompletion(keywords.at(index), m_keywordIcon);
+                m_completions << createCompletionItem(keywords.at(index), glslIcon(IconTypeKeyword));
 //            m_keywordVariant = languageVariant(m_interface->mimeType());
 //        }
 
@@ -376,50 +452,44 @@ IAssistProposal *GLSLCompletionAssistProcessor::perform(const IAssistInterface *
         if (var) {
             int storageType = var->qualifiers() & GLSL::QualifiedTypeAST::StorageMask;
             if (storageType == GLSL::QualifiedTypeAST::Attribute)
-                icon = m_attributeIcon;
+                icon = glslIcon(IconTypeAttribute);
             else if (storageType == GLSL::QualifiedTypeAST::Uniform)
-                icon = m_uniformIcon;
+                icon = glslIcon(IconTypeUniform);
             else if (storageType == GLSL::QualifiedTypeAST::Varying)
-                icon = m_varyingIcon;
+                icon = glslIcon(IconTypeVarying);
             else if (storageType == GLSL::QualifiedTypeAST::Const)
-                icon = m_constIcon;
+                icon = glslIcon(IconTypeConst);
             else
-                icon = m_varIcon;
+                icon = glslIcon(IconTypeVariable);
         } else if (s->asArgument()) {
-            icon = m_varIcon;
+            icon = glslIcon(IconTypeVariable);
         } else if (s->asFunction() || s->asOverloadSet()) {
-            icon = m_functionIcon;
+            icon = glslIcon(IconTypeFunction);
         } else if (s->asStruct()) {
-            icon = m_typeIcon;
+            icon = glslIcon(IconTypeType);
         } else {
-            icon = m_otherIcon;
+            icon = glslIcon(IconTypeOther);
         }
         if (specialMembers.contains(s->name()))
-            addCompletion(s->name(), icon, SpecialMemberOrder);
+            m_completions << createCompletionItem(s->name(), icon, SpecialMemberOrder);
         else
-            addCompletion(s->name(), icon);
+            m_completions << createCompletionItem(s->name(), icon);
     }
 
     m_startPosition = pos + 1;
-    return createContentProposal();
+
+    return new GenericProposal(m_startPosition, m_completions);
 }
 
-IAssistProposal *GLSLCompletionAssistProcessor::createContentProposal() const
-{
-    IGenericProposalModel *model = new BasicProposalItemListModel(m_completions);
-    IAssistProposal *proposal = new GenericProposal(m_startPosition, model);
-    return proposal;
-}
-
-IAssistProposal *GLSLCompletionAssistProcessor::createHintProposal(
+IAssistProposal *GlslCompletionAssistProcessor::createHintProposal(
     const QVector<GLSL::Function *> &symbols)
 {
-    IFunctionHintProposalModel *model = new GLSLFunctionHintProposalModel(symbols);
+    FunctionHintProposalModelPtr model(new GlslFunctionHintProposalModel(symbols));
     IAssistProposal *proposal = new FunctionHintProposal(m_startPosition, model);
     return proposal;
 }
 
-bool GLSLCompletionAssistProcessor::acceptsIdleEditor() const
+bool GlslCompletionAssistProcessor::acceptsIdleEditor() const
 {
     const int cursorPosition = m_interface->position();
     const QChar ch = m_interface->characterAt(cursorPosition - 1);
@@ -437,9 +507,10 @@ bool GLSLCompletionAssistProcessor::acceptsIdleEditor() const
         ++pos;
 
         const QString word = m_interface->textAt(pos, cursorPosition - pos);
-        if (word.length() > 2 && checkStartOfIdentifier(word)) {
-            for (int i = 0; i < word.length(); ++i) {
-                if (! isIdentifierChar(word.at(i)))
+        if (word.length() >= TextEditorSettings::completionSettings().m_characterThreshold
+                && checkStartOfIdentifier(word)) {
+            for (auto character : word) {
+                if (!isIdentifierChar(character))
                     return false;
             }
             return true;
@@ -449,28 +520,19 @@ bool GLSLCompletionAssistProcessor::acceptsIdleEditor() const
     return isActivationChar(ch);
 }
 
-void GLSLCompletionAssistProcessor::addCompletion(const QString &text,
-                                                  const QIcon &icon,
-                                                  int order)
-{
-    BasicProposalItem *item = new BasicProposalItem;
-    item->setText(text);
-    item->setIcon(icon);
-    item->setOrder(order);
-    m_completions.append(item);
-}
-
 // -----------------------------
-// GLSLCompletionAssistInterface
+// GlslCompletionAssistInterface
 // -----------------------------
-GLSLCompletionAssistInterface::GLSLCompletionAssistInterface(QTextDocument *textDocument,
-                                                             int position,
-                                                             const QString &fileName,
-                                                             TextEditor::AssistReason reason,
+GlslCompletionAssistInterface::GlslCompletionAssistInterface(const QTextCursor &cursor,
+                                                             const Utils::FilePath &fileName,
+                                                             AssistReason reason,
                                                              const QString &mimeType,
                                                              const Document::Ptr &glslDoc)
-    : DefaultAssistInterface(textDocument, position, fileName, reason)
+    : AssistInterface(cursor, fileName, reason)
     , m_mimeType(mimeType)
     , m_glslDoc(glslDoc)
 {
 }
+
+} // namespace Internal
+} // namespace GlslEditor
