@@ -1,28 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Dmitry Savchenko
-** Copyright (C) 2016 Vasiliy Sorokin
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 Dmitry Savchenko
+// Copyright (C) 2016 Vasiliy Sorokin
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
 
 #pragma once
 
@@ -60,7 +38,6 @@ public:
     QString displayName() const override;
     int priorityInStatusBar() const override;
     void clearContents() override;
-    void visibilityChanged(bool visible) override;
     void setFocus() override;
     bool hasFocus() const override;
     bool canFocus() const override;
@@ -80,8 +57,8 @@ private:
     void scopeButtonClicked(QAbstractButton *button);
     void todoTreeViewClicked(const QModelIndex &index);
     void updateTodoCount();
-    void updateFilter();
-    void clearFilter();
+    void updateKeywordFilter();
+    void clearKeywordFilter();
 
     void createTreeView();
     void freeTreeView();
